<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="2.0"
	xmlns="http://www.w3.org/1999/xhtml"
	xmlns:h="http://www.w3.org/1999/xhtml"
	xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
	xmlns:fn="http://www.w3.org/2005/xpath-functions"
	xmlns:svg="http://www.w3.org/2000/svg"
	xmlns:xs="http://www.w3.org/2001/XMLSchema"
	exclude-result-prefixes="fn h xs">
	<xsl:output 
		method="xml" 
		indent="yes" 
		encoding="UTF-8"		
		doctype-public="-//W3C//DTD XHTML 1.1 plus MathML 2.0 plus SVG 1.1//EN" 
		doctype-system="http://www.w3.org/2002/04/xhtml-math-svg/xhtml-math-svg.dtd"/>
	
		
	<xsl:param name="title"/>
	<xsl:param name="isRoot"/>
	<xsl:param name="urlBase"/>
	<xsl:param name="wwwRead"/>
	<xsl:param name="wwwPost"/>
	
	
	<!-- Celý dokument -->
	<xsl:template match="/">
		<html>
			<head>
				<meta http-equiv="Content-Type" content="application/xhtml+xml; charset=utf-8" />
				<meta http-equiv="X-NNTP-Generated" content="{fn:current-dateTime()}" />
				<xsl:if test="$urlBase">
					<base href="{$urlBase}"/>
				</xsl:if>
				<xsl:if test="$title">
					<title><xsl:value-of select="$title"/></title>
				</xsl:if>
				<style type="text/css">
					body {
						font-family: sans-serif;
						font-size: 16px;
					}
					
					.wwwLinks {
						border-top: 2px solid grey;
						margin-top: 16px;
						font-size: 66%;
					}
					
					blockquote {
						background-color: #eee;
						font-style: italic;
						padding: 2px 20px;
						margin: 10px 32px;
						border-left: 3px solid silver;
					}
					
					pre {
						background-color: #eee;
						padding: 2px 20px;
						margin: 10px 32px;
						border-left: 3px solid #a00;
					}
					
					img {
						margin: 8px;
					}
					
					a img {
						border: none;
					}
					
					/** TODO: smazat */
					.mešuge {
						background-color: #afa;
					}
				</style>
			</head>
			<body>
				<xsl:if test="$title and $isRoot">
					<h1><xsl:value-of select="$title"/></h1>			
				</xsl:if>
				<xsl:call-template name="zpracujTělo">
					<xsl:with-param name="prvek" select="h:html/h:body/node()[1]"/>
				</xsl:call-template>
				
				<xsl:if test="$wwwRead or $wwwPost">
					<div class="wwwLinks">
						<p>Přístup přes síť www:</p>
						<ul>
							<xsl:if test="$wwwRead"><li><a href="{$wwwRead}">číst tento příspěvek</a></li></xsl:if>
							<xsl:if test="$wwwPost"><li><a href="{$wwwPost}">odpovědět na tento příspěvek</a></li></xsl:if>
						</ul>
					</div>
				</xsl:if>
			</body>
		</html>
	</xsl:template>
	
	
	<!-- Odkazy -->
	<xsl:template match="h:a">
		<a href="{@href}" title="{@title}"><xsl:apply-templates/></a>
	</xsl:template>
	
	
	<!-- Obrázky -->
	<xsl:template match="h:img">
		<img src="{@src}" alt="{@alt}" title="{@title}"><xsl:apply-templates/></img>
	</xsl:template>
	
	<!-- Zkratky -->
	<xsl:template match="h:abbr">
		<abbr title="{@title}"><xsl:apply-templates/></abbr>
	</xsl:template>
	
	
	<!-- Další povolené značky – ostatní odfiltrujeme (zbude z nich jen text) -->
	<xsl:template match="*">
		<xsl:choose>		
			<xsl:when test="
				name() = 'h1' or
				name() = 'h2' or
				name() = 'h3' or
				name() = 'h4' or
				name() = 'h5' or
				name() = 'h6' or
				name() = 'p' or
				name() = 'div' or
				name() = 'br' or
				name() = 'strong' or
				name() = 'em' or
				name() = 'sub' or
				name() = 'sup' or
				name() = 'del' or
				name() = 'ul' or
				name() = 'ol' or
				name() = 'li' or
				name() = 'pre' or
				name() = 'code' or
				name() = 'blockquote'">
				<xsl:element name="{name()}">
					<xsl:apply-templates/>
				</xsl:element>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="."/>
			</xsl:otherwise>
		</xsl:choose>		
    </xsl:template>
    
    
    <!-- 
    	Z neuzavřeného (nevalidně se vyskytujícího v body) textu uděláme odstavce.
    -->
	<xsl:template name="zpracujTělo">
		<xsl:param name="prvek"/>
		<xsl:if test="$prvek">
			<li>
				&lt;
				<xsl:choose>
					<xsl:when test="$prvek/self::text()">
						<!--
							Textový uzel → budeme dělat odstavce
							(rekurzivně se opět zavolá šablona zpracujTělo)
						<xsl:call-template name="dělejOdstavce">
							<xsl:with-param name="blokTextu" select="."/>
						</xsl:call-template>
						-->
						[T]
					</xsl:when>
					<xsl:otherwise>
						<!--
							Jiný uzel → aplikujeme obecné šablony
							a zpracujeme následující uzel.
						<xsl:apply-templates select="."/>
						-->
						[N]
					</xsl:otherwise>					
				</xsl:choose>
				[<xsl:value-of select="$prvek/name()"/>]
				[<xsl:value-of select="$prvek/text()"/>]
				[<xsl:value-of select="$prvek/self::text()"/>]
				&gt;
			</li>	
			<xsl:call-template name="zpracujTělo">
					<xsl:with-param name="prvek" select="$prvek/following-sibling::node()[1]"/>
			</xsl:call-template>
				
		</xsl:if>
	</xsl:template>
	
	<xsl:template name="dělejOdstavce">
		<xsl:param name="blokTextu"/>
		<xsl:variable name="oddělovač" select="'\n\s+\n\s+'"/>
		<xsl:for-each select="fn:tokenize(., $oddělovač)">
			<xsl:if test="normalize-space(.)">
				<p class="mešuge">
					<xsl:value-of select="."/>
					<!-- 
						Toto je poslední odstavec bloku textu 
						a blok nekončí dvěma konci řádku → 
						může za ním následovat značka (např. odkaz nebo tučné písmo)
						vnořená do téhož odstavce
					-->
					<xsl:if test="
						position() = last() and
						not(fn:matches($blokTextu, concat('.*', $oddělovač ,'$')))
						">
						
						<xsl:variable name="n" select="$blokTextu/following-sibling::*[position() = 1]"/>
						<xsl:variable name="nn" select="$n/name()"/>
						<!--
							Za blokem textu nenásleduje značka, která nemůže být uvnitř odstavce.
						-->
						<xsl:if test="not(
								$nn = 'p' or
								$nn = 'div' or								
								$nn = 'h1' or								
								$nn = 'h2' or								
								$nn = 'h3' or								
								$nn = 'h4' or								
								$nn = 'h5' or								
								$nn = 'h6' or								
								$nn = 'pre' or								
								$nn = 'table' or								
								$nn = 'blockquote' or								
								$nn = 'hr'								
							)">
							<xsl:apply-templates select="$n"/>
							<xsl:apply-templates select="$n/following-sibling::text()[position() = 1]"/>
						</xsl:if>						
					</xsl:if>
				</p>
			</xsl:if>
		</xsl:for-each>
	</xsl:template>
    
	
</xsl:stylesheet>
